use std::io;
use std::time::{Duration, Instant};

use crate::constants::{HIGHLIGHT_COLOR, TICK_RATE};
use crate::ui_screens::utils::centered_rect_size;
use crate::ui_screens::ScreenResult;
use crossterm::event;
use crossterm::event::{Event, KeyCode};
use tui::backend::Backend;
use tui::style::*;
use tui::text::Text;
use tui::widgets::*;
use tui::{Frame, Terminal};

#[derive(Debug, Eq, PartialEq, Copy, Clone, Default)]
pub enum SelectPlayModeResult {
    #[default]
    PlayAgainstBot,
    PlayRandom,
    CreateInvite,
    AcceptInvite,
    Exit,
}

impl SelectPlayModeResult {
    /// Specify whether a selected play mode requires a user name or not
    pub fn need_player_name(&self) -> bool {
        self != &SelectPlayModeResult::PlayAgainstBot && self != &SelectPlayModeResult::Exit
    }

    /// Specify whether a selected play mode requires a the user to specify its own game rules or
    /// not
    pub fn need_custom_rules(&self) -> bool {
        self == &SelectPlayModeResult::PlayAgainstBot || self == &SelectPlayModeResult::CreateInvite
    }
}

#[derive(Debug, Clone)]
struct PlayModeDescription {
    name: &'static str,
    value: SelectPlayModeResult,
}

const AVAILABLE_PLAY_MODES: [PlayModeDescription; 5] = [
    PlayModeDescription {
        name: "🤖 Play against bot (offline)",
        value: SelectPlayModeResult::PlayAgainstBot,
    },
    PlayModeDescription {
        name: "🎲 Play against random player (online)",
        value: SelectPlayModeResult::PlayRandom,
    },
    PlayModeDescription {
        name: "➕ Create play invite (online)",
        value: SelectPlayModeResult::CreateInvite,
    },
    PlayModeDescription {
        name: "🎫 Accept play invite (online)",
        value: SelectPlayModeResult::AcceptInvite,
    },
    PlayModeDescription {
        name: "❌ Exit app",
        value: SelectPlayModeResult::Exit,
    },
];

#[derive(Default)]
pub struct SelectPlayModeScreen {
    state: ListState,
    curr_selection: usize,
}

impl SelectPlayModeScreen {
    pub fn show<B: Backend>(
        mut self,
        terminal: &mut Terminal<B>,
    ) -> io::Result<ScreenResult<SelectPlayModeResult>> {
        let mut last_tick = Instant::now();
        loop {
            self.state.select(Some(self.curr_selection));
            terminal.draw(|f| self.ui(f))?;

            let timeout = TICK_RATE
                .checked_sub(last_tick.elapsed())
                .unwrap_or_else(|| Duration::from_secs(0));

            if crossterm::event::poll(timeout)? {
                if let Event::Key(key) = event::read()? {
                    match key.code {
                        KeyCode::Char('q') => return Ok(ScreenResult::Canceled),
                        KeyCode::Enter => {
                            return Ok(ScreenResult::Ok(
                                AVAILABLE_PLAY_MODES[self.curr_selection].value,
                            ));
                        }
                        KeyCode::Down => self.curr_selection += 1,
                        KeyCode::Up => self.curr_selection += AVAILABLE_PLAY_MODES.len() - 1,
                        _ => {}
                    }

                    self.curr_selection %= AVAILABLE_PLAY_MODES.len();
                }
            }
            if last_tick.elapsed() >= TICK_RATE {
                last_tick = Instant::now();
            }
        }
    }

    fn ui<B: Backend>(&mut self, f: &mut Frame<B>) {
        let area = centered_rect_size(50, 2 + AVAILABLE_PLAY_MODES.len() as u16, &f.size());

        // Create a List from all list items and highlight the currently selected one
        let items = AVAILABLE_PLAY_MODES
            .iter()
            .map(|mode| ListItem::new(Text::raw(mode.name)))
            .collect::<Vec<_>>();
        let items = List::new(items)
            .block(
                Block::default()
                    .title("Select play mode")
                    .borders(Borders::ALL),
            )
            .highlight_style(
                Style::default()
                    .fg(HIGHLIGHT_COLOR)
                    .add_modifier(Modifier::BOLD),
            )
            .highlight_symbol(">> ");

        f.render_stateful_widget(items, area, &mut self.state);
    }
}
